#pragma TextEncoding = "UTF-8"
#pragma rtGlobals=3				// Use modern global access method and strict wave access
#pragma DefaultTab={3,20,4}		// Set default tab width in Igor Pro 9 and later

Menu "Load Waves"
	"Load EPL Cochlear File...", LoadEPLCochlearFile("", "", "")
End

static Function FindFirstDataLine(pathName, filePath)
	String pathName		// Name of symbolic path or ""
	String filePath		// Name of file or partial path relative to symbolic path.

	Variable refNum

	Open/R/P=$pathName refNum as filePath
	
	String buffer, text
	Variable line = 0
	
	do
		FReadLine refNum, buffer
		if (strlen(buffer) == 0)
			Close refNum
			return -1					// The expected keyword was not found in the file
		endif
		text = buffer[0,4]
		if (CmpStr(text, ":DATA") == 0)	 // Line does start with ":DATA" ?
			Close refNum
			return line + 1				 // Success: The next line is the first data line.
		endif
		line += 1
	while(1)
	
	return -1			// We will never get here
End

static Function/S ReadLevels(pathName, filePath)
	String pathName		// Name of symbolic path or ""
	String filePath		// Name of file or partial path relative to symbolic path.

	Variable refNum

	Open/R/P=$pathName refNum as filePath
	
	String buffer, outputText=""
	Variable line = 0

	String levelsPreface = ":LEVELS:"
	int levelsPrefaceLen = strlen(levelsPreface)
	
	do
		FReadLine refNum, buffer
		if (strlen(buffer) == 0)
			Close refNum
			return ""							// The expected keyword was not found in the file
		endif
		String prefix = buffer[0,levelsPrefaceLen-1]
		if (CmpStr(prefix, levelsPreface) == 0)		// Line does start with ":LEVELS" ?
			buffer = RemoveEnding(buffer, "\n")		// Remove LF, if any
			buffer = RemoveEnding(buffer, "\r")		// Remove CR, if any
			int len = strlen(buffer)
			outputText = buffer[levelsPrefaceLen,len]
			break
		endif
		line += 1
	while(1)
	
	// We have a line of text like ":LEVELS:20;25;30;35;40;45;50;55;60;65;70;75;80;"
	
	return outputText
End

// Returns string suitable for LoadWave /B flag
// e.g.
// Print GetColumnInfoStrFromLevelsText(":LEVELS:20;25;30;35;40;45;50;55;60;65;70;75;80;")
// (You need to remove "static" to execute that line.)
static Function/S GetColumnInfoStrFromLevelsText(levelsText)
	String levelsText	// e.g., ":LEVELS:20;25;30;35;40;45;50;55;60;65;70;75;80;"
	
	String columnInfoStr = ""
	int numLevels = ItemsInList(levelsText)
	int i
	for(i=0; i<numLevels; i+=1)
		String level = StringFromList(i, levelsText)
		String name = "LEVEL_" + level
		String nameText = "N='" + name + "';"
		columnInfoStr += nameText
	endfor
	
	return columnInfoStr
End

Function LoadEPLCochlearFile(pathName, filePath, extension)
	String pathName		// Name of symbolic path or "" to display dialog.
	String filePath		// Name of file or "" to display dialog. Can also be full or partial path relative to symbolic path.
	String extension	// e.g., ".tsv"
 
	Variable refNum
 
	// Possibly display Open File dialog.
	if ((strlen(pathName)==0) || (strlen(filePath)==0))
		String fileFilters = "TSV (*.tsv):.tsv;"
		fileFilters += "All Files:.*;"
		Open /D /R /P=$pathName /F=fileFilters refNum as filePath
		filePath = S_fileName			// S_fileName is set by Open/D
		if (strlen(filePath) == 0)		// User cancelled?
			return -1
		endif
		// filePath is now a full path to the file.
	endif
	
	String levelsText = ReadLevels(pathName, filePath)
	if (strlen(levelsText) == 0)
		Printf "Failed to find :LEVELS: line in file %s\r", filePath
		return -1				// Failed to find LEVELS line
	endif
	// We have a line of text like "20;25;30;35;40;45;50;55;60;65;70;75;80;"
	
	String columnInfoStr = GetColumnInfoStrFromLevelsText(levelsText)
	// e.g., columnInfoStr = "N=LEVEL_20;N=LEVEL_25;..."
	
	Variable firstDataLine = FindFirstDataLine(pathName, filePath)
	if (firstDataLine < 0)
		Printf "No data found in file %s\r", filePath
		return -1
	endif
 
	LoadWave /G /D /O /E=1 /K=0 /L={0,firstDataLine,0,0,0} /A /B=columnInfoStr /P=$pathName filePath
 
	return 0
End
